package fsmwizard

import (
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/internal/db/model"
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/internal/handler/iface"
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/pkg/fsm"
	"github.com/mymmrac/telego"
	tu "github.com/mymmrac/telego/telegoutil"
)

const RegisterStateID fsm.StateID = "register"

type RegisterState struct {
	State
}

func NewRegisterState(app iface.App) fsm.IState[Wizard] {
	return &RegisterState{newBase(app)}
}

func (s *RegisterState) Enter(pl *Wizard, _ fsm.MachineControls[Wizard]) error {
	if pl.Data.Message != nil && pl.Data.Message.Chat.Type != telego.ChatTypePrivate {
		return fsm.ErrPreventTransition
	}
	return nil
}

func (s *RegisterState) Handle(pl *Wizard, mc fsm.MachineControls[Wizard]) {
	loc := s.Localizer(pl.Data.Message.From.LanguageCode)
	userRepo := s.App.DB().ForUser()
	user, err := userRepo.ByTelegramID(pl.Data.Message.From.ID)
	if err != nil {
		return
	}

	if user != nil && user.ID > 0 {
		var shouldUpdate bool
		if user.ChatID != pl.Data.Message.Chat.ID {
			user.ChatID = pl.Data.Message.Chat.ID
			shouldUpdate = true
		}
		if user.Language != pl.Data.Message.From.LanguageCode {
			user.Language = pl.Data.Message.From.LanguageCode
			shouldUpdate = true
		}
		if shouldUpdate {
			s.LogError(userRepo.Save(user))
		}
		pl.User = user
		s.Move(mc, WaitingForMemberWebhookStateID, pl)
		return
	}

	err = userRepo.Save(&model.User{
		TelegramID: pl.Data.Message.From.ID,
		ChatID:     pl.Data.Message.Chat.ID,
		Language:   pl.Data.Message.From.LanguageCode,
	})
	if err != nil {
		s.LogError(err)
		_, err = s.App.TG().SendMessage(&telego.SendMessageParams{
			ChatID:    tu.ID(pl.Data.Message.Chat.ID),
			Text:      loc.Message("internal_error"),
			ParseMode: telego.ModeMarkdown,
		})
		s.LogError(err)
	}
	pl.User = user
	s.Move(mc, WaitingForMemberWebhookStateID, pl)
}

func (s *RegisterState) Exit(pl *Wizard) {
	if pl.Data.Message == nil {
		return
	}
	loc := s.Localizer(pl.Data.Message.From.LanguageCode)
	_, err := s.App.TG().SendMessage(&telego.SendMessageParams{
		ChatID:    tu.ID(pl.Data.Message.Chat.ID),
		Text:      loc.Message("welcome"),
		ParseMode: telego.ModeMarkdown,
	})
	s.LogError(err)
}

func (s *RegisterState) ID() fsm.StateID {
	return RegisterStateID
}
